﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Problems.DataAnalysis;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class AzzaliBenchmark2 : ArtificialRegressionDataDescriptor {
    public override string Name { get { return "Azzali Benchmark2 B2 = ((X3 + X1) / mean(X4)) * X2"; } }
    public override string Description { get { return "I. Azzali, L. Vanneschi, S. Silva, I. Bakurov, and M. Giacobini, “A Vectorial Approach to Genetic Programming,” EuroGP, pp. 213–227, 2019."; } }

    protected override string TargetVariable { get { return "B2"; } }
    protected override string[] VariableNames { get { return new string[] { "X1", "X2", "X3", "X4" }; } }
    protected override string[] AllowedInputVariables { get { return new string[] { "X1", "X2", "X3", "X4" }; } }
    protected override int TrainingPartitionStart { get { return 0; } }
    protected override int TrainingPartitionEnd { get { return 70; } }
    protected override int TestPartitionStart { get { return 70; } }
    protected override int TestPartitionEnd { get { return 100; } }

    public int Seed { get; private set; }

    public AzzaliBenchmark2()
      : this((int)DateTime.Now.Ticks) { }
    public AzzaliBenchmark2(int seed)
      : base() {
      Seed = seed;
    }


    protected override List<List<double>> GenerateValues() { return null; }
    protected override List<IList> GenerateValuesExtended() {
      var rand = new MersenneTwister((uint)Seed);

      var x1Column = new List<double>(100);
      var x2Column = new List<double>(100);
      var x3Column = new List<DoubleVector>(100);
      var x4Column = new List<DoubleVector>(100);
      var b2Column = new List<DoubleVector>(100);

      for (int i = 0; i < 100; i++) {
        var x1 = rand.NextDouble(-10, +10);
        var x2 = rand.NextDouble(-10, +10);
        var x3 = rand.NextDoubleVector(10, 40, 10);
        var x4 = rand.NextDoubleVector(-5, +5, 10);

        var b2 = ((x3 + x1) / x4.CumulativeMean()) * x2;

        x1Column.Add(x1);
        x2Column.Add(x2);
        x3Column.Add(x3);
        x4Column.Add(x4);
        b2Column.Add(b2);
      }

      return new List<IList> { x1Column, x2Column, x3Column, x4Column, b2Column };
    }
  }
}